var sqliteParser = require('sqlite-parser');
var cli = require('cli');
var options = cli.parse({
 contentdir: [null, 'Allows to specify arbitrary content directory.', 'string', 'content'],
 sql: ['q', 'SQL querry', 'string']
});

var ast = sqliteParser(options.sql).statement[0];

console.log(JSON.stringify(ast,null,'  '));

var getProp = function(prop, article) {
  var ret = article;
  prop.split('.').forEach(function(p) {
    if(ret === undefined) return;
    ret = ret[p];
  });
  return ret;
}

var functions = {
  'contains': function(context, args) {
    if(args.length !== 2) throw new Error('Wrong number of arguments for function contains');
    var prop = evalExpr(context, args[0]);
    if(!Array.isArray(prop)) return false;
    var val = evalExpr(context, args[1]);
    var ret = false;
    prop.forEach(function(el) {
      if(el == val) ret = true;
    })
    return ret;
  },
  'count': function(context, args) {
    if(args.length !== 1) throw new Error('Wrong number of arguments for function count');
    var prop = evalExpr(context, args[0]);
    if(prop === undefined) return 0;
    if(!Array.isArray(prop)) return 1;
    return prop.length;
  }
}

var binaryOperators = {
  'and': function(context, left, right) {
    return evalExpr(context, left) && evalExpr(context, right);
  },
  'or': function(context, left, right) {
    return evalExpr(context, left) || evalExpr(context, right);
  },
  '==': function(context, left, right) {
    return evalExpr(context, left) == evalExpr(context, right);
  },
  '>': function(context, left, right) {
    return evalExpr(context, left) > evalExpr(context, right);
  },
  '<': function(context, left, right) {
    return evalExpr(context, left) < evalExpr(context, right);
  },
  '>=': function(context, left, right) {
    return evalExpr(context, left) >= evalExpr(context, right);
  },
  '<=': function(context, left, right) {
    return evalExpr(context, left) <= evalExpr(context, right);
  },
  '!=': function(context, left, right) {
    return evalExpr(context, left) != evalExpr(context, right);
  },
}

var literals = {
  'decimal': function(context, value) {
    return Number(value);
  },
  'string': function(context, value) {
    return value;
  }
}

var evalExpr = function(context, expr) {
  if(expr.type == 'identifier') {
    return getProp(expr.name, context);
  } else if(expr.type == 'expression' && expr.format == 'binary') {
    if(typeof binaryOperators[expr.operation] === 'function') {
      return binaryOperators[expr.operation](context, expr.left, expr.right);
    } else {
      throw new Error('Unsupported binary operator '+expr.operation);
    }
  } else if(expr.type == 'function') {
    if(typeof functions[expr.name] === 'function') {
      return functions[expr.name](context, expr.args);
    } else {
      throw new Error('Unsupported function '+expr.name);
    }
  } else if(expr.type == 'literal') {
    if(typeof literals[expr.variant] === 'function') {
      return literals[expr.variant](context, expr.value);
    } else {
      throw new Error('Unsupported literal variant '+expr.variant);
    }
  } else {
    throw new Error('Unsupported expression '+JSON.stringify(expr));
  }
  throw new Error('Code should not get here. This is really bad.');
}

var jobs = require('./sitegin/jobs');
jobs.registerMultiple(
  {},
  ['parseHugo', './parseHugo'],
  ['readFiles', './readFiles']
)
.then(function() {
  return jobs.run('readFiles',{config:{
    sourceDir: options.contentdir,
    articlesLocation: 'articles',
    redirectsLocation: 'redirects'
  }})
})
.then(function(obj) {
  return jobs.run('parseHugo',obj)
})
.then(function(obj){
  return obj.pages
})
.then(function(pages) {
  return pages.filter(function(page) {
    return evalExpr(page, ast.where[0]);
  })
})
.then(function(pages) {
  var results = ast.result;
  if(results[0].variant == 'star') {
    pages.forEach(function(page) {
      console.log(JSON.stringify(page));
    })
    return;
  }
  pages.forEach(function(page) {
    var n = {};
    results.forEach(function(result) {
      if(result.alias) n[result.alias] = getProp(result.name, page);
      else n[result.name] = getProp(page, result.name);
    })
    console.log(n);
  })
})
.then(function() {
  process.exit(0);
})
.catch(function(e) {
  console.log(e.stack);
  process.exit(1);
})
